// Copyright (C) 2013 - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

// Reference
// "Polynômes de chaos", Jean-Marc Martinez
// Decompose the function 
//
// Y=g(X) 
//
// where X is a random variable into:
// Y=a0*P0(x)+a1*P1(x)+...+ad*Pd(x)
//
// where Pi are orthogonal polynomials.

function y=LegendreWeight(x)
    // Returns w(x)
    y=1
endfunction

function y=LegendreWeightIntegral()
    // Returns int_{-1}^1 w(x) dx
    y=2
endfunction

function y=LegendrePoly(n)
    // Create a Legendre polynomial of degree n
    if (n==0) then
        y=poly(1,"x","coeff")
    elseif (n==1) then
        y=poly([0 1],"x","coeff")
    else
        polyx=poly([0 1],"x","coeff")
        // y(n-2)
        yn2=poly(1,"x","coeff")
        // y(n-1)
        yn1=polyx
        for k=2:n
            y=((2*k-1)*polyx*yn1-(k-1)*yn2)/k
            yn2=yn1
            yn1=y
        end
    end
endfunction

function y=LegendreNorm(n)
    // Returns Int Pn(x)^2 w(x) dx
    n=n(:)
    y=2 ./(2*n+1)
endfunction

function y=HermitePoly(n)
    // Create a Hermite polynomial of degree n
    if (n==0) then
        y=poly(1,"x","coeff")
    elseif (n==1) then
        y=poly([0 1],"x","coeff")
    else
        polyx=poly([0 1],"x","coeff")
        // y(n-2)
        yn2=poly(1,"x","coeff")
        // y(n-1)
        yn1=polyx
        for k=2:n
            y=polyx*yn1-(k-1)*yn2
            yn2=yn1
            yn1=y
        end
    end
endfunction

function y=HermiteNorm(n)
    // Returns Int Hn(x)^2 w(x) dx
    n=n(:)
    y=sqrt(2*%pi)*factorial(n)
endfunction

function y=myg(x)
    y=exp(x)
endfunction

function y=integralForHermitePC(x,myfunction,i)
    // Evaluates the numerator integral for 1D PC decomposition
    // on Hermite polynomials
    P = HermitePoly(i)
    Px = horner(P,x)
    y = Px*myfunction(x)
endfunction

function y=integralForLegendrePC(x,myfunction,i)
    // Evaluates the numerator integral for 1D PC decomposition
    // on Legendre polynomials
    // Computes 
    // g(x)*Li(x)*f(x)
    // where 
    // g : the model to approximate
    // Li : Legendre polynomial of order i
    // f : Legendre probability distribution function
    // In fact, f(x)=w(x)/I
    // where I=integral_{-1}^1 w(x) dx
    
    P = LegendrePoly(i)
    Px = horner(P,x)
    gx = myfunction(x)
    fx = LegendreWeight(x)
    y = Px.*gx.*fx
endfunction

function y=evalPCLegendre(x,a)
    // Evaluates the PC decomposition at point x
    d=size(a,"*")-1
    P=poly(0,"x","coeff")
    for i=1:d+1
        P=P+a(i)*LegendrePoly(i-1)
    end
    y = horner(P,x)
endfunction

// Suppose that X is uniform in [-1,1] > Legendre polynomials
d=7;
for i=1:d+1
    a(i)=intg(-1,1,list(integralForLegendrePC,myg,i-1));
    a(i)=a(i)/LegendreNorm(i-1)
end

myg(0.5)
evalPCLegendre(0.5,a)

// Show a degree 7 PC
x=linspace(-1,1,100);
y0=evalPCLegendre(x,a(1));
y1=evalPCLegendre(x,a(1:2));
y2=evalPCLegendre(x,a(1:3));
y3=evalPCLegendre(x,a(1:4));
y5=evalPCLegendre(x,a(1:6));
y7=evalPCLegendre(x,a(1:8));
scf();
// d=0
subplot(2,2,1)
plot(x,myg(x),"r-")
plot(x,y0,"b-")
xtitle("Legendre PC - d=0","x","");
legend(["g(x)","d=0"]);
// d=1
subplot(2,2,2)
plot(x,myg(x),"r-")
plot(x,y1,"b-")
xtitle("Legendre PC - d=1","x","");
legend(["g(x)","d=1"]);
// d=2
subplot(2,2,3)
plot(x,myg(x),"r-")
plot(x,y2,"b-")
xtitle("Legendre PC - d=2","x","");
legend(["g(x)","d=2"]);
// d=3
subplot(2,2,4)
plot(x,myg(x),"r-")
plot(x,y3,"b-")
xtitle("Legendre PC - d=3","x","");
legend(["g(x)","d=3"]);
//
// Evaluate the mean, the variance
x=grand(1000000,1,"unf",-1,1);
y=myg(x);
// Mean
disp(mean(y)) // 1.1750574
mupc=a(1)
disp(a(1))    // 1.1752012
// Variance
disp(variance(y)) // 0.4319607
sigmapc=sqrt(sum(a(2:8).^2 .*LegendreNorm(1:7)))
sigmapc^2 // 0.8646647 (Does not work = x2 coeff !!!)

//////////////////////////////////
// Check orthogonality
function y=orthocheck(x,i,j)
    Pi = LegendrePoly(i)
    Pix = horner(Pi,x)
    Pj = LegendrePoly(j)
    Pjx = horner(Pj,x)/sqrt(LegendreNorm(j))
    y=Pix*Pjx
endfunction

intg(-1,1,list(orthocheck,0,0))
intg(-1,1,list(orthocheck,0,1))
intg(-1,1,list(orthocheck,0,2))
intg(-1,1,list(orthocheck,0,3))
intg(-1,1,list(orthocheck,0,4))

intg(-1,1,list(orthocheck,1,1))
intg(-1,1,list(orthocheck,1,2))
intg(-1,1,list(orthocheck,1,3))
intg(-1,1,list(orthocheck,1,4))

intg(-1,1,list(orthocheck,2,1))
intg(-1,1,list(orthocheck,2,2))
intg(-1,1,list(orthocheck,2,3))
intg(-1,1,list(orthocheck,2,4))

