// Copyright (C) 2013 - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

// Reference
// http://en.wikipedia.org/wiki/Gaussian_integral
// http://en.wikipedia.org/wiki/Hermite_polynomials

///////////////////////////////////
// Gaussian weight function

function y=mygauss(x)
    y=exp(-0.5*x.^2)
endfunction

x=linspace(-4,4,1000);
y=mygauss(x);
scf();
plot(x,y)
xtitle("Gaussian weight","x","w(x)")

[v,err]=intg(-10,10,mygauss)
sqrt(2*%pi)

///////////////////////////////////
// Gaussian distribution function

function y=mygaussf(x)
    y=exp(-0.5*x.^2)/sqrt(2*%pi)
endfunction

x=linspace(-4,4,1000);
y=mygaussf(x);
scf();
plot(x,y)
xtitle("Gaussian distribution","x","f(x)")

///////////////////////////////////

function y=Hermite1W(x)
    y=x.^2 .*exp(-0.5*x.^2)
endfunction

x=linspace(-4,4,1000);
y=Hermite1W(x);
scf();
plot(x,y)
xtitle("","x","$x^2w(x)$")

[v,err]=intg(-10,10,Hermite1W)
sqrt(2*%pi)

///////////////////////////////////

function y=Hermite2W(x)
    y=(x.^2-1).^2 .*exp(-0.5*x.^2)
endfunction

x=linspace(-4,4,1000);
y=Hermite2W(x);
scf();
plot(x,y)
xtitle("","x","$(x^2-1)^2w(x)$")

[v,err]=intg(-10,10,Hermite2W)
sqrt(2*%pi)*2

///////////////////////////////////
//
// Evaluate Hermite Polynomials
//

function y=HermiteSlow(x,n)
    // Evaluate a Hermite polynomial of degree n at x
    if (n==0) then
        y=1
    elseif (n==1) then
        y=x
    else
        yyy=HermiteSlow(x,n-2)
        yy=HermiteSlow(x,n-1)
        y=x.*yy-(n-1)*yyy
    end
    disp([y,n])
endfunction

function y=Hermite(x,n)
    // Evaluate a Hermite polynomial of degree n at x
    if (n==0) then
        y=1
    elseif (n==1) then
        y=x
    else
        // y(n-2)
        yn2=1
        // y(n-1)
        yn1=x
        for k=2:n
            y=x.*yn1-(k-1)*yn2
            yn2=yn1
            yn1=y
        end
    end
endfunction

x=linspace(-3,3,100);
y0=Hermite(x,0);
y1=Hermite(x,1);
y2=Hermite(x,2);
y3=Hermite(x,3);
h=scf();
plot(x,y0,"r-")
plot(x,y1,"g-")
plot(x,y2,"b-")
plot(x,y3,"c:")
xtitle("Hermite Polynomials","x","He(x)")
legend(["n=0","n=1","n=2","n=3"]);
h.children.data_bounds(1:2,2)=[-5;5];
h.children.children(4).children.thickness=3;
h.children.children(5).children.thickness=2;
h.children.children(5).children.line_style=5;

///////////////////////////////////
//
// Evaluate Hermite Polynomials
//

function y=HermitePolySlow(n)
    // Create a Hermite polynomial of degree n
    if (n==0) then
        y=poly(1,"x","coeff")
    elseif (n==1) then
        y=poly([0 1],"x","coeff")
    else
        polyx=poly([0 1],"x","coeff")
        yyy=HermitePolySlow(n-2)
        yy=HermitePolySlow(n-1)
        y=polyx*yy-(n-1)*yyy
    end
endfunction

function y=HermitePoly(n)
    // Create a Hermite polynomial of degree n
    if (n==0) then
        y=poly(1,"x","coeff")
    elseif (n==1) then
        y=poly([0 1],"x","coeff")
    else
        polyx=poly([0 1],"x","coeff")
        // y(n-2)
        yn2=poly(1,"x","coeff")
        // y(n-1)
        yn1=polyx
        for k=2:n
            y=polyx*yn1-(k-1)*yn2
            yn2=yn1
            yn1=y
        end
    end
endfunction

for n=0:10
    y=HermitePoly(n);
    disp(y)
end

///////////////////////////////////
//
// Check that int Hn(x)^2 w(x) dx=sqrt(2pi) n!
// for w(x)=exp(-x^2/2)
//

function y=sqHew(x,n)
    y=Hermite(x,n)^2*exp(-x^2/2)
endfunction

function y=HermiteNorm(n)
    // Returns Int Hn(x)^2 w(x) dx
    y=sqrt(2*%pi)*factorial(n)
endfunction

for n = 1:5
    [v,err]=intg(-10,10,list(sqHew,n));
    disp([n,HermiteNorm(n),v])
end

